@@ -5,21 +5,19 @@ import logging
 import threading
 import uuid
 from asyncio import Lock
-from typing import Any, Mapping, Optional
+from typing import Mapping, Optional
 from urllib.parse import urlparse
 
 import paho.mqtt.client as mqtt
 
-from roborock.api import SPECIAL_COMMANDS, RoborockClient, md5hex
-from roborock.exceptions import CommandVacuumError, RoborockException, VacuumError
-
+from .api import SPECIAL_COMMANDS, RoborockClient, md5hex, KEEPALIVE
 from .containers import RoborockDeviceInfo, UserData
-from .roborock_message import RoborockMessage, RoborockParser, md5bin
+from .exceptions import CommandVacuumError, RoborockException, VacuumError
 from .roborock_future import RoborockFuture
+from .roborock_message import RoborockMessage, RoborockParser, md5bin
 from .typing import RoborockCommand
 
 _LOGGER = logging.getLogger(__name__)
-MQTT_KEEPALIVE = 60
 CONNECT_REQUEST_ID = 0
 DISCONNECT_REQUEST_ID = 1
 
@@ -50,8 +48,6 @@ class RoborockMqttClient(RoborockClient, mqtt.Client):
         self._endpoint = base64.b64encode(md5bin(rriot.k)[8:14]).decode()
         self._waiting_queue: dict[int, RoborockFuture] = {}
         self._mutex = Lock()
-        self._last_device_msg_in = mqtt.time_func()
-        self._last_disconnection = mqtt.time_func()
         self.update_client_id()
 
     def __del__(self) -> None:
@@ -81,7 +77,6 @@ class RoborockMqttClient(RoborockClient, mqtt.Client):
 
     def on_message(self, *args, **kwargs) -> None:
         _, __, msg = args
-        self._last_device_msg_in = mqtt.time_func()
         device_id = msg.topic.split("/").pop()
         messages, _ = RoborockParser.decode(msg.payload, self.devices_info[device_id].device.local_key)
         super().on_message(messages)
@@ -89,11 +84,9 @@ class RoborockMqttClient(RoborockClient, mqtt.Client):
     def on_disconnect(self, *args, **kwargs) -> None:
         try:
             _, __, rc, ___ = args
-            self._last_disconnection = mqtt.time_func()
-            message = f"Roborock mqtt client disconnected (rc: {rc})"
+            super().on_disconnect(RoborockException(f"(rc: {rc})"))
             if rc == mqtt.MQTT_ERR_PROTOCOL:
                 self.update_client_id()
-            _LOGGER.warning(message)
             connection_queue = self._waiting_queue.get(DISCONNECT_REQUEST_ID)
             if connection_queue:
                 connection_queue.resolve((True, None))
@@ -103,18 +96,10 @@ class RoborockMqttClient(RoborockClient, mqtt.Client):
     def update_client_id(self):
         self._client_id = mqtt.base62(uuid.uuid4().int, padding=22)
 
-    def _async_check_keepalive(self) -> None:
-        now = mqtt.time_func()
-        # noinspection PyUnresolvedReferences
-        if (
-            now - self._last_disconnection > self._keepalive**2  # type: ignore[attr-defined]
-            and now - self._last_device_msg_in > self._keepalive  # type: ignore[attr-defined]
-        ):
-            self._ping_t = self._last_device_msg_in
-
     def _check_keepalive(self) -> None:
-        self._async_check_keepalive()
-        # noinspection PyUnresolvedReferences
+        if not self.should_keepalive():
+            self._ping_t = self.time_func() - KEEPALIVE
+            # noinspection PyUnresolvedReferences
         super()._check_keepalive()  # type: ignore[misc]
 
     def sync_stop_loop(self) -> None:
@@ -138,34 +123,30 @@ class RoborockMqttClient(RoborockClient, mqtt.Client):
         return rc == mqtt.MQTT_ERR_SUCCESS
 
     def sync_connect(self) -> bool:
-        rc = mqtt.MQTT_ERR_AGAIN
         self.sync_start_loop()
         if not self.is_connected():
             if self._mqtt_port is None or self._mqtt_host is None:
                 raise RoborockException("Mqtt information was not entered. Cannot connect.")
             _LOGGER.info("Connecting to mqtt")
-            rc = super().connect(host=self._mqtt_host, port=self._mqtt_port, keepalive=MQTT_KEEPALIVE)
-            if rc != mqtt.MQTT_ERR_SUCCESS:
-                raise RoborockException(f"Failed to connect (rc:{rc})")
-        return rc == mqtt.MQTT_ERR_SUCCESS
+            super().connect_async(host=self._mqtt_host, port=self._mqtt_port, keepalive=KEEPALIVE)
+            return True
+        return False
 
-    async def async_disconnect(self) -> Any:
+    async def async_disconnect(self) -> None:
         async with self._mutex:
             disconnecting = self.sync_disconnect()
             if disconnecting:
-                (response, err) = await self._async_response(DISCONNECT_REQUEST_ID)
+                (_, err) = await self._async_response(DISCONNECT_REQUEST_ID)
                 if err:
                     raise RoborockException(err) from err
-                return response
 
-    async def async_connect(self) -> Any:
+    async def async_connect(self) -> None:
         async with self._mutex:
             connecting = self.sync_connect()
             if connecting:
-                (response, err) = await self._async_response(CONNECT_REQUEST_ID)
+                (_, err) = await self._async_response(CONNECT_REQUEST_ID)
                 if err:
                     raise RoborockException(err) from err
-                return response
 
     async def validate_connection(self) -> None:
         await self.async_connect()
@@ -195,4 +176,8 @@ class RoborockMqttClient(RoborockClient, mqtt.Client):
         return response
 
     async def get_map_v1(self, device_id):
-        return await self.send_command(device_id, RoborockCommand.GET_MAP_V1)
+        try:
+            return await self.send_command(device_id, RoborockCommand.GET_MAP_V1)
+        except RoborockException as e:
+            _LOGGER.error(e)
+        return None
